<?php
/*
 * Plugin Name: wpForo - Blog Cross Posting
 * Plugin URI: http://wpforo.com
 * Description: Blog to Forum and Forum to Blog content synchronization. Blog posts with Forum topics and Blog comments with Forum replies. 
 * Version: 1.1.8
 * Author: gVectors Team (G. Zakaryan, H. Martirosyan)
 * Contributors: A. Chakhoyan, R. Hovhannisyan
 * Author URI: https://gvectors.com/
 * Text Domain: wpforo_cross
 * Domain Path: /languages/
 */

if (!defined('ABSPATH')) {
    exit();
}

include_once 'includes/wpForoCrossPostingConst.php';
include_once 'includes/wpForoCrossPostingOptions.php';
include_once 'includes/wpForoCrossPostingDbManager.php';
include_once 'includes/gvt-api-manager.php';

class wpForoCrossPosting {

    private $options;
    private $db;
    public $version;
    public $isGoodbyeCaptchaActive;
    public $goodbyeCaptchaTocken = '';

    public function __construct() {
        add_action('plugins_loaded', array(&$this, 'pluginLoad'));
    }

    public function pluginLoad() {
        if (function_exists('WPF')) {
            $this->options = new wpForoCrossPostingOptions();
            $this->db = new wpForoCrossPostingDbManager();
            add_action('add_meta_boxes', array(&$this, 'addMetabox'));
            add_filter('wpforo_plugins_tabs_array_filter', array(&$this, 'addOptionsTab'));
            add_filter('wpforo_plugins_option_files_array_filter', array(&$this, 'addOptionsFilePath'));
            /* ==== POST ==== */
            add_action('wp_insert_post', array(&$this, 'savePost'), 10, 2);
            add_action('transition_post_status', array(&$this, 'transitionPostStatus'), 12, 3);
            add_action('before_delete_post', array(&$this, 'deletePost'));
            add_filter('the_content', array(&$this, 'postContent'), 9999);
            add_action('wpforo_after_move_topic', array(&$this, 'moveTopic'), 154, 2);
            /* ==== COMMENT ==== */
            add_action('comment_post', array(&$this, 'addComment'), 10, 3);
            add_action('transition_comment_status', array(&$this, 'changeCommentStatus'), 99, 3);
            add_action('edit_comment', array(&$this, 'editComment'), 99, 2);
            add_action('delete_comment', array(&$this, 'deleteComment'));
            add_filter('comment_text', array(&$this, 'commentText'), 9999, 2);
            /* ==== FORUM POST ==== */
            add_action('wpforo_after_add_post', array(&$this, 'addForumPost'), 99, 2);
            add_action('wpforo_after_edit_post', array(&$this, 'editForumPost'), 99, 2);
            add_action('wpforo_after_delete_post', array(&$this, 'deleteForumPost'));
            /* ==== CONTENT INFO ===== */
            add_action('wpforo_tpl_post_loop_after_content', array(&$this, 'crossPostingInfo'), 99, 2);
            add_action('wp_enqueue_scripts', array(&$this, 'frontEndStylesScripts'));
            add_action('admin_init', array(&$this, 'checkVersion'));
            add_action('wpforo_editor_post_submit_after', array(&$this, 'printGoodbyeCaptchaField'));
            add_action('wpforo_portable_editor_post_submit_after', array(&$this, 'printGoodbyeCaptchaField'));
            load_plugin_textdomain('wpforo_cross', false, dirname(plugin_basename(__FILE__)) . '/languages/');
            new GVT_API_Manager(__FILE__, 'wpforo-settings&tab=plugins&subtab=cross_posting', 'wpforo_settings_page_top');
            $this->initGoodbyeCaptchaField();
        }
    }

    private function initGoodbyeCaptchaField() {
        $this->isGoodbyeCaptchaActive = is_callable(array('GdbcWordPressPublicModule', 'isCommentsProtectionActivated')) && GdbcWordPressPublicModule::isCommentsProtectionActivated();
        if ($this->isGoodbyeCaptchaActive) {
            $this->goodbyeCaptchaTocken = GdbcWordPressPublicModule::getInstance()->getTokenFieldHtml();
        }
    }

    public function printGoodbyeCaptchaField() {
        echo $this->goodbyeCaptchaTocken;
    }

    public function addMetabox($postType) {
        if (in_array($postType, $this->options->postTypes)) {
            add_meta_box(
                    'wpforo_forums_list', __('Blog to Forum Cross Posting', 'wpforo_cross'), array(&$this, 'theForumsList'), $postType, 'side', 'high'
            );
            if ($this->options->crossPostingContent == wpForoCrossPostingOptions::OPTION_CROSPOSTING_CONTENT_CUSTOM) {
                add_meta_box(
                        'wpforo_crossposting_custom_content', __('Cross-posting Custom Content', 'wpforo_cross'), array(&$this, 'theCustomContent'), $postType, 'advanced', 'high'
                );
            }
        }
    }

    public function theForumsList($post) {
        $forumID = get_post_meta($post->ID, wpForoCrossPostingOptions::META_POST_FORUM_ID, true);
        $isCrossPostingDisabled = get_post_meta($post->ID, wpForoCrossPostingOptions::META_POST_CROSPOSTING_DISABLED, true);
        ?>
        <?php if ($isCrossPostingDisabled || ($forumID && $isCrossPostingDisabled)): ?>
            <?php if ($forumID): ?>
                <p><strong><?php _e('Blog Post to Forum Topic Cross-posting', 'wpforo_cross'); ?></strong></p>
                <p style="margin:5px 0px 5px 0px;"><?php _e('Awaiting for cross-posting.', 'wpforo_cross'); ?></p>
                <p style="margin:5px 0px 5px 0px;"><?php _e('This post will be cross-posted once the status is set to Published', 'wpforo_cross'); ?></p>
            <?php else : ?>
                <?php ?>
                <p><strong><?php _e('Blog Post to Forum Topic Cross-posting', 'wpforo_cross'); ?></strong></p>
                <p><?php _e('You can TURN ON this post to forum cross posting by selecting the target forum in this drop-down menu. Once this post is submitted or updated it\'ll be automatically posted or updated in selected forum as topic with the same title and content.', 'wpforo_cross'); ?></p>
                <?php $forumID = 0; ?>
            <?php endif; ?>
            <select id="parent" name="<?php echo wpForoCrossPostingOptions::WPFORO_FORUM_ID; ?>" class="postform">
                <option value="0"><?php _e('Cross Posting is Off', 'wpforo_cross'); ?> &nbsp;</option>
                <?php WPF()->forum->tree('select_box', FALSE, array($forumID)); ?>
            </select>
        <?php else: ?>
            <?php
            $topicID = get_post_meta($post->ID, wpForoCrossPostingOptions::META_POST_TOPIC_ID, true);
            $topicURL = '';
            if ($topicID) {
                $topicURL = wpforo_topic($topicID, 'url');
            }
            if ($topicURL):
                ?>
                <p style="margin:5px 0px 5px 0px;"><?php _e('Syncronization Options:', 'wpforo_cross'); ?>
                <ul style="padding:0px; margin:0px 0px 10px 15px;">
                    <li style="list-style:disc; line-height:16px; margin-bottom:2px; color:<?php echo ( $this->options->postCrossingEdit ) ? '#009900' : '#dd0000'; ?>"><?php _e('Update cross-posted Topic', 'wpforo_cross'); ?> <?php echo ( $this->options->postCrossingEdit ) ? '(on)' : '(off)'; ?></li>
                    <li style="list-style:disc; line-height:16px; margin-bottom:3px; color:<?php echo ( $this->options->postCrossingDelete ) ? '#009900' : '#dd0000'; ?>"><?php _e('Delete cross-posted Topic', 'wpforo_cross'); ?> <?php echo ( $this->options->postCrossingDelete ) ? '(on)' : '(off)'; ?></li>
                </ul>
                </p>
                <input type="hidden" name="<?php echo wpForoCrossPostingOptions::WPFORO_FORUM_ID; ?>" value="<?php echo intval($forumID); ?>" style="vertical-align:bottom;" />
                <p><label><input type="checkbox" name="wpfcp_turn_off" value="1" style="vertical-align:bottom;" /> <?php _e('Turn off Cross Posting', 'wpforo_cross'); ?></label></p>
                <p><a href="<?php echo esc_url($topicURL); ?>" target="_blank"><?php _e('View cross-posted topic', 'wpforo_cross'); ?> &raquo;</a></p>
            <?php else: ?>
                <p><strong><?php _e('Blog Post to Forum Topic Cross-posting', 'wpforo_cross'); ?></strong></p>
                <p><?php _e('You can TURN ON this post to forum cross posting by selecting the target forum in this drop-down menu. Once this post is submitted or updated it\'ll be automatically posted or updated in selected forum as topic with the same title and content.', 'wpforo_cross'); ?></p>
                <select id="parent" name="<?php echo wpForoCrossPostingOptions::WPFORO_FORUM_ID; ?>" class="postform">
                    <option value="0"><?php _e('Cross Posting is Off', 'wpforo_cross'); ?> &nbsp;</option>
                    <?php WPF()->forum->tree('select_box', FALSE, array($forumID)); ?>
                </select>
            <?php endif; ?>
        <?php endif; ?>

        <?php
    }

    public function theCustomContent($post) {
        $content = get_post_meta($post->ID, wpForoCrossPostingOptions::META_CROSPOSTING_CONTENT, true);
        wp_editor($content, wpForoCrossPostingOptions::META_CROSPOSTING_CONTENT, array('textarea_rows' => 10));
    }

    /* ============ POST ===================== */

    public function savePost($postID, $post) {
        if (wp_is_post_revision($postID) || !in_array($post->post_type, $this->options->postTypes)) {
            return;
        }
        $postStatus = $post->post_status;
        $action = filter_input(INPUT_POST, 'action', FILTER_SANITIZE_STRING);
        $customContent = isset($_POST[wpForoCrossPostingOptions::META_CROSPOSTING_CONTENT]) ? $_POST[wpForoCrossPostingOptions::META_CROSPOSTING_CONTENT] : '';
        $customContent = trim($customContent);

        if ($this->options->crossPostingContent == wpForoCrossPostingOptions::OPTION_CROSPOSTING_CONTENT_CUSTOM && $customContent) {
            update_post_meta($postID, wpForoCrossPostingOptions::META_CROSPOSTING_CONTENT, $customContent);
        }
        //EDIT next line changed by Excellent. Force code to run when metadata META_POST_CROSPOSTING_DISABLED is set (by client). Original: if ($action != 'inline-save') {	
        if (!metadata_exists( 'post', $postID, wpForoCrossPostingOptions::META_POST_CROSPOSTING_DISABLED) && $action != 'inline-save') {		
			$turnOff = filter_input(INPUT_POST, 'wpfcp_turn_off', FILTER_SANITIZE_NUMBER_INT);
            $forumID = filter_input(INPUT_POST, wpForoCrossPostingOptions::WPFORO_FORUM_ID, FILTER_SANITIZE_NUMBER_INT);
            $savedForumID = get_post_meta($postID, wpForoCrossPostingOptions::META_POST_FORUM_ID, true);
            if ($turnOff) {
                update_post_meta($postID, wpForoCrossPostingOptions::META_POST_CROSPOSTING_DISABLED, 1);
            }
        } else {
            $turnOff = get_post_meta($postID, wpForoCrossPostingOptions::META_POST_CROSPOSTING_DISABLED, true);
            $forumID = get_post_meta($postID, wpForoCrossPostingOptions::META_POST_FORUM_ID, true);
			$savedForumID = null;
        }
        if ($forumID && !$turnOff) {
			update_post_meta($postID, wpForoCrossPostingOptions::META_POST_FORUM_ID, $forumID);
            $topicID = get_post_meta($postID, wpForoCrossPostingOptions::META_POST_TOPIC_ID, true);
            $topic = WPF()->topic->get_topic($topicID);
            if ($postStatus == 'publish') {
                if ($topic) {
                    $this->editTopic($post, $forumID, $topicID);
                } else {
					$this->addTopic($post, $forumID);
					//EDIT next if statement added by Excellent. Set status to 0 (approved) when called from any xmlrpc client
					if (strpos(wp_debug_backtrace_summary(), 'wp_xmlrpc_server') !== false) {
						WPF()->db->query(
							"UPDATE IGNORE ".WPF()->tables->topics." 
							SET status = 0 
							WHERE topicid = " . get_post_meta($post->ID, wpForoCrossPostingOptions::META_POST_TOPIC_ID, true)
						);
					}
                }
				//EDIT next line commented out by Excellent. META_POST_CROSPOSTING_DISABLED controlled by client.
                //update_post_meta($postID, wpForoCrossPostingOptions::META_POST_CROSPOSTING_DISABLED, 0);
				//EDIT next  statement added by Excellent. Set private to 0 (public post) when post status = publish
				WPF()->db->query(
					"UPDATE IGNORE ".WPF()->tables->topics." 
					SET private = 0 
					WHERE topicid = " . get_post_meta($post->ID, wpForoCrossPostingOptions::META_POST_TOPIC_ID, true)
				);
                $this->synchComments($postID);
            } elseif ($postStatus == 'draft') {
                update_post_meta($postID, wpForoCrossPostingOptions::META_POST_CROSPOSTING_DISABLED, 1);
				//EDIT next  statement added by Excellent. Set private to 1 (private post) when post status = draft
				WPF()->db->query(
					"UPDATE IGNORE ".WPF()->tables->topics." 
					SET private = 1 
					WHERE topicid = " . get_post_meta($post->ID, wpForoCrossPostingOptions::META_POST_TOPIC_ID, true)
				);
            } elseif ($postStatus == 'trash') {
                WPF()->moderation->post_unapprove($topic['first_postid']);
                WPF()->notice->clear();
            }
            if ($savedForumID && $forumID && $forumID != $savedForumID && $topic && $this->options->postCrossingEdit) {
                WPF()->topic->move($topicID, $forumID);
            }
            WPF()->notice->clear();
        }
		//EDIT next if statement added by Excellent. Use SQL to open / close the topic based on META_POST_CROSPOSTING_DISABLED 
		if (metadata_exists('post', $postID, wpForoCrossPostingOptions::META_POST_CROSPOSTING_DISABLED) && metadata_exists('post', $postID, wpForoCrossPostingOptions::META_POST_TOPIC_ID)) {
			WPF()->db->query(
				"UPDATE IGNORE ".WPF()->tables->topics." 
				SET closed = " . get_post_meta($post->ID, wpForoCrossPostingOptions::META_POST_CROSPOSTING_DISABLED, true)." 
				WHERE topicid = " . get_post_meta($post->ID, wpForoCrossPostingOptions::META_POST_TOPIC_ID, true)
			);
		}
    }

    public function transitionPostStatus($new_status, $old_status, $post) {
        $forumID = get_post_meta($post->ID, wpForoCrossPostingOptions::META_POST_FORUM_ID, true);
        $turnOff = get_post_meta($post->ID, wpForoCrossPostingOptions::META_POST_CROSPOSTING_DISABLED, true);
        $action = filter_input(INPUT_POST, 'action', FILTER_SANITIZE_STRING);
        if (!$forumID || $turnOff) {
            return;
        }
        if ($old_status == 'future' && $new_status == 'publish') {
            $userID = $this->initCurrentUser();
            $topicID = get_post_meta($post->ID, wpForoCrossPostingOptions::META_POST_TOPIC_ID, true);
            $topic = WPF()->topic->get_topic($topicID);
            if ($topic) {
                $this->editTopic($post, $forumID, $topicID);
            } else {
                $this->addTopic($post, $forumID);
            }
			//EDIT next line commented out by Excellent. META_POST_CROSPOSTING_DISABLED controlled by client.
			//update_post_meta($post->ID, wpForoCrossPostingOptions::META_POST_CROSPOSTING_DISABLED, 0);
            $this->synchComments($post->ID);
            if ($userID) {
                wp_set_current_user(0);
            }
            WPF()->notice->clear();
        } else if ($new_status == 'trash') {
            $topicID = get_post_meta($post->ID, wpForoCrossPostingOptions::META_POST_TOPIC_ID, true);
            $topic = WPF()->topic->get_topic($topicID);
            if ($topic) {
                WPF()->moderation->post_unapprove($topic['first_postid']);
                WPF()->notice->clear();
            }
        } else if ($new_status != 'publish' && $old_status == 'publish') {
			//EDIT next line commented out by Excellent. META_POST_CROSPOSTING_DISABLED controlled by client.
			//update_post_meta($post->ID, wpForoCrossPostingOptions::META_POST_CROSPOSTING_DISABLED, 1);
        }
    }

    private function synchComments($postID) {
        if ($this->options->commentCrossingAdd && $postID) {
            $comments = get_comments(array('post_id' => $postID, 'orderby' => 'comment_date', 'order' => 'ASC', 'status' => 'approve'));
            if ($comments) {
                foreach ($comments as $comment) {
                    $ref = get_comment_meta($comment->comment_ID, wpForoCrossPostingOptions::META_COMMENT_SOURCE, true);
                    if (!$ref) {
                        $this->addComment($comment->comment_ID, 1, (array) $comment);
                    }
                }
            }

            $topicID = get_post_meta($postID, wpForoCrossPostingOptions::META_POST_TOPIC_ID, true);
            if ($topicID) {
                $replies = WPF()->post->get_posts(array('topicid' => $topicID, 'is_first_post' => 0));
                array_shift($replies);
                if ($replies) {
                    foreach ($replies as $reply) {
                        $blogCommentId = $this->db->getBlogCommentID($reply['postid']);
                        if (!$blogCommentId) {
                            $topic = WPF()->topic->get_topic($topicID);
                            $this->addForumPost($reply, $topic);
                        }
                    }
                }
            }
        }
    }

    private function addTopic($post, $forumID) {
        $args = $this->getRequestData($forumID, $post);
		if ($args) {
			$newTopicID = WPF()->topic->add($args);
			if ($newTopicID) {
				update_post_meta($post->ID, wpForoCrossPostingOptions::META_POST_FORUM_ID, $forumID);
				update_post_meta($post->ID, wpForoCrossPostingOptions::META_POST_TOPIC_ID, $newTopicID);
			}
			
		}
    }

    private function editTopic($post, $forumID, $topicID) {
		if (!$this->options->postCrossingEdit) {
            return false;
        }
        $args = $this->getRequestData($forumID, $post);
        if ($args && $topicID) {
            $args['topicid'] = $topicID;
            WPF()->topic->edit($args);
        }
    }

    private function getRequestData($forumID, $post) {
        $args = array(
            'forumid' => $forumID,
            'title' => $post->post_title,
            'userid' => $post->post_author,
            'body' => $this->crosspostingContent($post),
            'created' => $post->post_date_gmt,
            'slug' => $post->post_name
        );
        return $args;
    }

    private function crosspostingContent($post) {
        $content = $post->post_content;
        if ($this->options->crossPostingContent == wpForoCrossPostingOptions::OPTION_CROSPOSTING_CONTENT_EXCERPT) {
            $excerpt = $post->post_excerpt;
            if (trim($excerpt)) {
                $content = $excerpt;
            } else {
                $excerpt_length = apply_filters('excerpt_length', 55);
                $content = wp_trim_words($content, $excerpt_length);
            }
        } elseif ($this->options->crossPostingContent == wpForoCrossPostingOptions::OPTION_CROSPOSTING_CONTENT_CUSTOM) {
            $customContent = get_post_meta($post->ID, wpForoCrossPostingOptions::META_CROSPOSTING_CONTENT, true);
            if (trim($customContent)) {
                $content = $customContent;
            }
        }
        return $content;
    }

    public function deletePost($postID) {
        if ($this->options->postCrossingDelete) {
            $forumID = get_post_meta($postID, wpForoCrossPostingOptions::META_POST_FORUM_ID, true);
            $topicID = get_post_meta($postID, wpForoCrossPostingOptions::META_POST_TOPIC_ID, true);
            $isCrossPostingDisabled = get_post_meta($postID, wpForoCrossPostingOptions::META_POST_CROSPOSTING_DISABLED, true);
            if ($forumID && $topicID && !$isCrossPostingDisabled) {
                WPF()->topic->delete($topicID);
                WPF()->notice->clear();
            }
        }
    }

    public function postContent($content) {
        if ($this->options->showPostTopicRel && is_singular()) {
            global $post;
            $postID = $post->ID;
            $topicID = get_post_meta($postID, wpForoCrossPostingOptions::META_POST_TOPIC_ID, true);
            $isCrossPostingDisabled = get_post_meta($postID, wpForoCrossPostingOptions::META_POST_CROSPOSTING_DISABLED, true);
            if ($topicID && !$isCrossPostingDisabled) {
                $linkText = wpforo_phrase('This article is also published as a forum topic here', FALSE);
                if ($this->options->crossPostingContent != wpForoCrossPostingOptions::OPTION_CROSPOSTING_CONTENT_FULL) {
                    $linkText = wpforo_phrase('More discussion', FALSE);
                }
                $topicURL = wpforo_topic($topicID, 'url');
                $content .= '<div class="wpfcp-article-info">';
                $content .= '<a href="' . esc_url($topicURL) . '">' . $linkText . ' &raquo;</a>';
                $content .= '</div><div class="wpfcpcb"></div>';			
            }
        }
        return $content;
    }

    /* ============ COMMENT ===================== */

    public function addComment($commentID, $commentApproved, $data) {
        if ($this->options->commentCrossingAdd && $commentApproved === 1 && !isset($data['referrer'])) {
            $comment = $this->arrayToObject($data);
            $this->_addComment($commentID, $comment);
        }
    }

    public function changeCommentStatus($newStatus, $oldStatus, $comment) {
        $commentID = $comment->comment_ID;
        $forumPostID = get_comment_meta($commentID, wpForoCrossPostingOptions::META_COMMENT_FORUM_POST_ID, TRUE);
        if ($newStatus == 'approved' && $this->options->commentCrossingAdd && !$forumPostID) {
            $this->_addComment($commentID, $comment);
        }
    }

    private function _addComment($commentID, $comment) {
        if ($comment->comment_type) {
            return;
        }
        $args = $this->buildBlogCommentArgs($comment);
        if ($args) {
            $forumPostID = WPF()->post->add($args);
            if ($forumPostID) {
                add_comment_meta($commentID, wpForoCrossPostingOptions::META_COMMENT_FORUM_POST_ID, $forumPostID);
                add_comment_meta($commentID, wpForoCrossPostingOptions::META_COMMENT_SOURCE, wpForoCrossPostingOptions::META_COMMENT_SOURCE_BLOG);
            }
            WPF()->notice->clear();
        }
    }

    public function editComment($commentID, $data) {
        $forumPostID = get_comment_meta($commentID, wpForoCrossPostingOptions::META_COMMENT_FORUM_POST_ID, true);
        $actionSource = get_comment_meta($commentID, wpForoCrossPostingOptions::META_COMMENT_ACTION_SOURCE, true);
        if ($actionSource == wpForoCrossPostingOptions::META_COMMENT_SOURCE_FORUM) {
            delete_comment_meta($commentID, wpForoCrossPostingOptions::META_COMMENT_ACTION_SOURCE);
            return;
        }
        if ($forumPostID && $this->options->commentCrossingEdit) {
            $comment = $this->arrayToObject($data);
            $args = $this->buildBlogCommentArgs($comment);
            if ($args) {
                $args['postid'] = $forumPostID;
                update_comment_meta($commentID, wpForoCrossPostingOptions::META_COMMENT_ACTION_SOURCE, wpForoCrossPostingOptions::META_COMMENT_SOURCE_BLOG);
                WPF()->post->edit($args);
                WPF()->notice->clear();
            }
        }
    }

    public function deleteComment($commentID) {
        $forumPostID = get_comment_meta($commentID, wpForoCrossPostingOptions::META_COMMENT_FORUM_POST_ID, true);
        $forumPost = WPF()->post->get_post($forumPostID);
        if ($forumPost && $forumPostID && $this->options->commentCrossingDelete) {
            $comment = get_comment($commentID);
            $args = $this->buildBlogCommentArgs($comment);
            if ($args) {
                update_comment_meta($commentID, wpForoCrossPostingOptions::META_COMMENT_ACTION_SOURCE, wpForoCrossPostingOptions::META_COMMENT_SOURCE_BLOG);
                WPF()->post->delete($forumPostID);
                WPF()->notice->clear();
            }
        }
    }

    private function buildBlogCommentArgs($comment) {
        $args = array();
        if (!$comment) {
            return $args;
        }
        $postID = $comment->comment_post_ID;
        $blogPost = get_post($postID);
        $commentParent = $comment->comment_parent;
        $commentContent = $comment->comment_content;
        $userID = isset($comment->user_id) ? $comment->user_id : 0;
        $userName = $comment->comment_author;
        $userEmail = $comment->comment_author_email;
        $topicID = get_post_meta($postID, wpForoCrossPostingOptions::META_POST_TOPIC_ID, true);
        $forumID = get_post_meta($postID, wpForoCrossPostingOptions::META_POST_FORUM_ID, true);
        $forumPostParent = get_comment_meta($commentParent, wpForoCrossPostingOptions::META_COMMENT_FORUM_POST_ID, true);
        $isCrossPostingDisabled = get_post_meta($postID, wpForoCrossPostingOptions::META_POST_CROSPOSTING_DISABLED, true);
        if ($topicID && $forumID && !$isCrossPostingDisabled) {
            $args['forumid'] = $forumID;
            $args['topicid'] = $topicID;
            $args['title'] = __('RE:') . $blogPost->post_title;
            $args['body'] = $commentContent;
            $args['parentid'] = $forumPostParent ? $forumPostParent : 0;
            $args['userid'] = $userID;
            $args['name'] = $userName;
            $args['email'] = $userEmail;
            $args['referrer'] = wpForoCrossPostingOptions::META_COMMENT_SOURCE_BLOG;
            $args['created'] = $comment->comment_date_gmt;
        }
        return $args;
    }

    public function commentText($commentText, $comment = null) {
        $html = '';
        if (!is_string($commentText)) {
            return $commentText;
        }
        if ($comment && is_singular()) {
            $forumPostID = get_comment_meta($comment->comment_ID, wpForoCrossPostingOptions::META_COMMENT_FORUM_POST_ID, true);
            $isCrossPostingDisabled = get_post_meta($comment->comment_post_ID, wpForoCrossPostingOptions::META_POST_CROSPOSTING_DISABLED, true);
            $url = wpforo_post($forumPostID, 'url');
            if ($forumPostID && $url && !$isCrossPostingDisabled) {
                WPF()->current_object['forumid'] = get_post_meta($comment->comment_post_ID, wpForoCrossPostingOptions::META_POST_FORUM_ID, true);
                WPF()->current_object['forum'] = wpforo_forum(WPF()->current_object['forumid']);
                $commentText = wpforo_content_filter($commentText);
                if ($this->options->showCommentReplyRel) {
                    $html .= '<div class="wpfcp-comment-info" title="' . wpforo_phrase('This comment is also posted as a reply in related forum topic.', FALSE) . '">';
                    $html .= '<a href="' . esc_url($url) . '">' . wpforo_phrase('forum reply', FALSE) . ' &raquo;</a>';
                    $html .= '</div><div class="wpfcpcb"></div>';
                }
            }
        }
        return $commentText . $html;
    }

    /* ==== FORUM POST ==== */

    public function addForumPost($post, $topic) {
        if (!isset($post['referrer']) && $this->options->forumPostCrossingAdd) {
            $blogPostID = $this->db->getBlogPostID($topic['topicid']);
            if ($blogPostID) {
                $commentParent = 0;
                $userDisplayName = '';
                $userEmail = '';
                if ($post['userid']) {
                    $user = get_user_by('ID', $post['userid']);
                    $userDisplayName = $user->display_name;
                    $userEmail = $user->user_email;
                } elseif ($post['name'] && $post['email']) {
                    $userDisplayName = $post['name'];
                    $userEmail = $post['email'];
                } else {
                    return;
                }

                if ($post['parentid']) {
                    $commentParent = $this->db->getBlogCommentID($post['parentid']);
                }
                $data = array(
                    'comment_post_ID' => $blogPostID,
                    'comment_author' => $userDisplayName,
                    'comment_author_email' => $userEmail,
                    'comment_author_url' => '',
                    'comment_content' => $post['body'],
                    'comment_parent' => $commentParent,
                    'user_ID' => $post['userid'],
                    'comment_type' => '',
                    'referrer' => wpForoCrossPostingOptions::META_COMMENT_SOURCE_FORUM
                );
                $commentID = wp_new_comment($data);
                if ($commentID) {
                    add_comment_meta($commentID, wpForoCrossPostingOptions::META_COMMENT_FORUM_POST_ID, $post['postid']);
                    add_comment_meta($commentID, wpForoCrossPostingOptions::META_COMMENT_SOURCE, wpForoCrossPostingOptions::META_COMMENT_SOURCE_FORUM);
                }
            }
        }
    }

    public function editForumPost($filteredArgs) {
        if ($this->options->forumPostCrossingEdite) {
            $blogPostID = $this->db->getBlogPostID($filteredArgs['topicid']);
            $commentID = $this->db->getBlogCommentID($filteredArgs['postid']);
            $actionSource = get_comment_meta($commentID, wpForoCrossPostingOptions::META_COMMENT_ACTION_SOURCE, true);
            if ($blogPostID && $commentID && $actionSource != wpForoCrossPostingOptions::META_COMMENT_SOURCE_BLOG) {
                update_comment_meta($commentID, wpForoCrossPostingOptions::META_COMMENT_ACTION_SOURCE, wpForoCrossPostingOptions::META_COMMENT_SOURCE_FORUM);
                $commentArgs = array('comment_ID' => $commentID,
                    'comment_content' => $filteredArgs['body'],
                    'referrer' => wpForoCrossPostingOptions::META_COMMENT_SOURCE_FORUM);
                wp_update_comment($commentArgs);
            } else {
                delete_comment_meta($commentID, wpForoCrossPostingOptions::META_COMMENT_ACTION_SOURCE);
            }
        }
    }

    public function moveTopic($topic, $forumid) {
        $topicID = $topic['topicid'];
        $postID = $this->db->getBlogPostID($topicID);
        if ($postID) {
            update_post_meta($postID, wpForoCrossPostingOptions::META_POST_FORUM_ID, $forumid);
        }
    }

    public function deleteForumPost($post) {
        if ($this->options->forumPostCrossingDelete) {
            $forumPostID = $post['postid'];
            $blogPostID = $this->db->getBlogPostID($post['topicid']);
            if ($blogPostID) {
                $blogCommentID = $this->db->getBlogCommentID($forumPostID);
                $actionSource = get_comment_meta($blogCommentID, wpForoCrossPostingOptions::META_COMMENT_ACTION_SOURCE, true);
                if ($actionSource != wpForoCrossPostingOptions::META_COMMENT_SOURCE_BLOG) {
                    wp_delete_comment($blogCommentID);
                }
                $this->deleteBlogPostMeta($blogPostID, $post);
            }
        }
    }

    private function deleteBlogPostMeta($blogPostID, $post) {
        if ($post['is_first_post']) {
            delete_post_meta($blogPostID, wpForoCrossPostingOptions::META_POST_FORUM_ID);
            delete_post_meta($blogPostID, wpForoCrossPostingOptions::META_POST_TOPIC_ID);
        }
    }

    public function addOptionsTab($tabs) {
        $tabs[wpForoCrossPostingOptions::OPTIONS_TAB] = __('Cross Posting', 'wpforo_cross');
        return $tabs;
    }

    public function addOptionsFilePath($tabFiles) {
        $tabFiles[wpForoCrossPostingOptions::OPTIONS_TAB] = plugin_dir_path(__FILE__) . 'includes/options.php';
        return $tabFiles;
    }

    public function getOptions() {
        return $this->options;
    }

    private function arrayToObject($array) {
        if (!is_array($array)) {
            return $array;
        }
        $object = new stdClass();
        if (is_array($array) && count($array) > 0) {
            foreach ($array as $name => $value) {
                $name = trim($name);
                if (!empty($name)) {
                    $object->$name = $this->arrayToObject($value);
                }
            }
            return $object;
        } else {
            return FALSE;
        }
    }

    public function crossPostingInfo($post, $member) {
        if (!isset($post['is_first_post']))
            return false;
        if ($post['is_first_post']) {
            $id = $post['topicid'];
            $type = 'topic';
        } else {
            $id = $post['postid'];
            $type = 'reply';
        }
        $crossURL = $this->crossURL($id, $type);
        if (isset($id) && $crossURL) {
            if ($post['is_first_post'] && $crossURL && $this->options->showTopicPostRel):
                $linkText = __('This topic is also published as an article here', 'wpforo_cross');
                if ($this->options->crossPostingContent != wpForoCrossPostingOptions::OPTION_CROSPOSTING_CONTENT_FULL) {
                    $linkText = wpforo_phrase('Read more', FALSE);
                }
                ?>
                <div class="wpfcp-topic-info"><a href="<?php echo esc_url($crossURL); ?>"><?php wpforo_phrase($linkText); ?> &raquo;</a></div><div class="wpfcpcr"></div>
            <?php elseif (!$post['is_first_post'] && $this->options->showReplyCommentRel): ?>
                <div class="wpfcp-post-info" title="<?php wpforo_phrase('This reply is also posted as a comment under related article.'); ?>"><a href="<?php echo esc_url($crossURL); ?>"><i class="far fa-comment"></i>&nbsp;&nbsp;<?php wpforo_phrase('comment'); ?></a></div><div class="wpfcpcr"></div>
                <?php
            endif;
        }
    }

    public function crossURL($id, $type = 'topic') {
        if (!$id)
            return false;
        $wpforo = WPF();
        if ($type == 'topic') {
            $sql = $wpforo->db->prepare("SELECT `post_id` FROM `{$wpforo->db->postmeta}` WHERE `meta_key` = %s AND `meta_value` = %s", wpForoCrossPostingOptions::META_POST_TOPIC_ID, $id);
            $postID = $wpforo->db->get_var($sql);
            $isCrossPostingDisabled = get_post_meta($postID, wpForoCrossPostingOptions::META_POST_CROSPOSTING_DISABLED, true);
            if ($postID && !$isCrossPostingDisabled) {
                return get_permalink($postID);
            }
        } elseif ($type == 'reply') {
            $sql = $wpforo->db->prepare("SELECT `comment_id` FROM `{$wpforo->db->commentmeta}` WHERE `meta_key` = %s AND `meta_value` = %s", wpForoCrossPostingOptions::META_COMMENT_FORUM_POST_ID, $id);
            $commentID = $wpforo->db->get_var($sql);
            $isCrossPostingDisabled = true;
            if ($commentID) {
                $comment = get_comment($commentID);
                $isCrossPostingDisabled = get_post_meta($comment->comment_post_ID, wpForoCrossPostingOptions::META_POST_CROSPOSTING_DISABLED, true);
            }
            if ($commentID && !$isCrossPostingDisabled) {
                return get_comment_link($commentID);
            }
        }
        return false;
    }

    public function frontEndStylesScripts() {
        wp_register_style('wpforo-cross-css', plugins_url('/assets/css/wpforo-cross.css', __FILE__));
        wp_enqueue_style('wpforo-cross-css');
        if (is_rtl()) {
            wp_register_style('wpforo-cross-rtl-css', plugins_url('/assets/css/wpforo-cross-rtl.css', __FILE__));
            wp_enqueue_style('wpforo-cross-rtl-css');
        }
    }

    public function checkVersion() {
        $pluginData = get_plugin_data(__FILE__);
        $this->version = '1.0.0';
        if (version_compare($pluginData['Version'], $this->version, '>')) {
            if ($this->version === '1.0.0') {
                add_option(wpForoCrossPostingOptions::PLUGIN_VERSION, $pluginData['Version']);
            } else {
                update_option(wpForoCrossPostingOptions::PLUGIN_VERSION, $pluginData['Version']);
            }
        }
    }

    private function initCurrentUser() {
        $userID = 0;
        if (!WPF()->current_user['ID']) {
            $adminUser = WPF()->member->get_members(array('groupid' => 1, 'row_count' => 1));
            $userID = $adminUser[0]['ID'];
            wp_set_current_user($adminUser[0]['ID'], $adminUser[0]['user_login']);
            WPF()->member->init_current_user();
        }
        return $userID;
    }

}

$wpForoCrossPosting = new wpForoCrossPosting();